#ifndef JACOBI_HH
#define JACOBI_HH

namespace Jacobi {

  typedef double   valueType ; // the type for real number
  typedef unsigned indexType ; // the type for integer (unsigned)

  void
  GivensLeft( indexType const n,        /* Number of rows of matrix A             */
              valueType       A[],      /* Pointer to coefficients of A           */
              indexType const lda,      /* Leading dimension of A, Fortran style  */
              indexType const row1,     /* first row for Givens rotation          */
              indexType const row2,     /* second row for Givens rotation         */
              valueType const theta ) ;
  void
  GivensRight( indexType const n,        /* Number of rows of matrix A             */
               valueType       A[],      /* Pointer to coefficients of A           */
               indexType const lda,      /* Leading dimension of A, Fortran style  */
               indexType const col1,     /* first column for Givens rotation       */
               indexType const col2,     /* second column for Givens rotation      */
               valueType const theta ) ;
  
  valueType
  evalAngle( indexType const n,    /* Number of rows of matrix A             */
             valueType       A[],  /* Pointer to coefficients of A           */
             indexType const lda,  /* Leading dimension of A, Fortran style  */
             indexType const row,
             indexType const col ) ;

  bool
  Jacobi( indexType const n,         /* Number of rows of matrix A             */
          valueType       A[],       /* Pointer to coefficients of A           */
          indexType const lda,       /* Leading dimension of A, Fortran style  */
          valueType       Q[],       /* Pointer to coefficients of Q           */
          indexType const ldq,       /* Leading dimension of Q, Fortran style  */
          valueType const epsi,      /* tolerance for off diagonal elements    */
          indexType const maxIter ); /* maximum number of big iterations       */
  
}

#endif